"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfiguration = exports.globFiles = exports.possibleJSONStringToArray = void 0;
const node_assert_1 = require("node:assert");
const debug_1 = __importDefault(require("debug"));
const camelcase_1 = __importDefault(require("camelcase"));
const nconf_1 = __importDefault(require("nconf"));
const dotenv_1 = __importDefault(require("dotenv"));
const deepmerge_1 = __importDefault(require("@fastify/deepmerge"));
const node_fs_1 = __importDefault(require("node:fs"));
const glob_1 = require("glob");
const path_1 = __importDefault(require("path"));
const secure_json_parse_1 = __importDefault(require("secure-json-parse"));
const debug = (0, debug_1.default)('ms-conf');
const { hasOwnProperty } = Object.prototype;
const { isArray } = Array;
const { env } = process;
const verbose = hasOwnProperty.call(env, 'DOTENV_NOT_SILENT') === false;
const cwd = process.cwd();
const merge = (0, deepmerge_1.default)({
    mergeArray: () => (_, source) => {
        return source;
    }
});
// safe json parse
function parseJSONSafe(possibleJSON) {
    try {
        return (0, secure_json_parse_1.default)(possibleJSON);
    }
    catch (e) {
        return possibleJSON;
    }
}
// make camelCase keys
const camelCaseKeys = (camelize) => function processKeys(obj, value, key) {
    const camelized = camelize ? key : (0, camelcase_1.default)(key);
    if (value == null) {
        obj[camelized] = value;
        return obj;
    }
    if (typeof value === 'object') {
        const gatherer = obj[camelized] = Object.create(null);
        for (const [innerKey, innerValue] of Object.entries(value)) {
            processKeys(gatherer, innerValue, innerKey);
        }
    }
    else if (typeof value === 'string') {
        obj[camelized] = parseJSONSafe(value);
    }
    else {
        obj[camelized] = value;
    }
    return obj;
};
// read file from path and try to parse it
const readFileFactory = (crashOnError) => (configuration, absPath) => {
    (0, node_assert_1.strict)(path_1.default.isAbsolute(absPath), `${absPath} must be an absolute path`);
    try {
        // delete loaded file
        delete require.cache[absPath];
        debug('loading %s', absPath);
        // eslint-disable-next-line @typescript-eslint/no-var-requires
        return merge(configuration, require(absPath));
    }
    catch (err) {
        if (crashOnError) {
            throw err;
        }
        else {
            process.stderr.write(`Failed to include file ${absPath}, err: ${err.message}\n`);
            return configuration;
        }
    }
};
function possibleJSONStringToArray(filePaths) {
    let files;
    try {
        files = JSON.parse(filePaths);
    }
    catch (e) {
        files = [filePaths];
    }
    if (!isArray(files)) {
        throw new Error('NCONF_FILE_PATH must be a stringified array or a string');
    }
    return files;
}
exports.possibleJSONStringToArray = possibleJSONStringToArray;
function resolve(filePath) {
    return require.resolve(filePath);
}
function resolveAbsPaths(paths) {
    const absolutePaths = paths.reduce((resolvedPaths, filePath) => {
        const stats = node_fs_1.default.statSync(filePath);
        if (stats.isFile()) {
            resolvedPaths.push(resolve(filePath));
        }
        else if (stats.isDirectory()) {
            // NOTE: can be improved
            // this is an extra call, but we dont care since it's a one-time op
            const absPaths = (0, glob_1.sync)(`${filePath}/*.{ts,js,cjs,json}`)
                .map(resolve)
                .filter((x) => x.endsWith('.d.ts') === false);
            resolvedPaths.push(...absPaths);
        }
        return resolvedPaths;
    }, []);
    return Array.from(new Set(absolutePaths));
}
function globFiles(prependFile, filePaths = [], configuration = Object.create(null), crashOnError = true) {
    // if we get parsed JSON array - use it right away
    const files = isArray(filePaths)
        ? filePaths
        : possibleJSONStringToArray(filePaths);
    if (typeof prependFile === 'string') {
        files.unshift(prependFile);
    }
    // prepare merger
    const mergeFile = readFileFactory(crashOnError);
    // resolve paths and merge
    for (const file of resolveAbsPaths(files)) {
        configuration = mergeFile(configuration, file);
    }
    return configuration;
}
exports.globFiles = globFiles;
function loadConfiguration(crashOnError, prependFile, appendConfiguration) {
    // load dotenv
    const dotenvConfig = {
        verbose,
        encoding: env.DOTENV_ENCODING || 'utf-8',
        path: env.DOTENV_FILE_PATH || `${cwd}/.env`,
    };
    // load dotenv and report error
    const result = dotenv_1.default.config(dotenvConfig);
    if (result.error) {
        debug('failed to load %s due to', dotenvConfig.path, result.error);
    }
    // do we camelize?
    const camelize = hasOwnProperty.call(env, 'NCONF_NO_CAMELCASE');
    const namespaceKey = env.NCONF_NAMESPACE;
    const filePaths = env.NCONF_FILE_PATH;
    // if we don't have it yet
    (0, node_assert_1.strict)(namespaceKey, 'NCONF_NAMESPACE must be present in order to parse your configuration');
    // init nconf store
    nconf_1.default.use('memory');
    nconf_1.default.reset();
    nconf_1.default.argv();
    nconf_1.default.env({
        separator: env.NCONF_SEPARATOR || '__',
        match: hasOwnProperty.call(env, 'NCONF_MATCH') ? new RegExp(env.NCONF_MATCH, env.NCONF_MATCH_OPTS) : null,
        whitelist: hasOwnProperty.call(env, 'NCONF_WHITELIST') ? JSON.parse(env.NCONF_WHITELIST) : null,
    });
    // pull camelCase data
    const namespace = nconf_1.default.get(namespaceKey);
    (0, node_assert_1.strict)(typeof namespace === 'object' && namespace !== null && !Array.isArray(namespace), 'namespace must be a js object');
    const configFromEnv = Object.create(null);
    const normalizer = camelCaseKeys(camelize);
    for (const [key, value] of Object.entries(namespace)) {
        normalizer(configFromEnv, value, key);
    }
    let config = Object.create(null);
    if (filePaths || prependFile) {
        // nconf file does not merge configuration, it will either omit it
        // or overwrite it, since it's JSON and is already parsed, what we will
        // do is pass it into configuration as is after it was already camelCased and merged
        //
        // nconf.file(env.NCONF_FILE_PATH);
        debug('globbing files', prependFile, filePaths);
        config = globFiles(prependFile, filePaths, config, crashOnError);
        debug('result\n%j', config);
    }
    debug('merging env\n%j', configFromEnv);
    config = merge(config, configFromEnv);
    debug('result\n%j', config);
    if (appendConfiguration !== undefined) {
        debug('appending config: %j', appendConfiguration);
        config = merge(config, appendConfiguration);
        debug('result: %j', config);
    }
    debug('prepared config\n%j', config);
    return config;
}
exports.loadConfiguration = loadConfiguration;
//# sourceMappingURL=load-config.js.map